@@ -44,6 +44,7 @@ from . import psglobals
 from . import SimulationParameters as SP
 from .ExpressionAST import ExpressionAST
 from .Node import Node
+from .Average import AverageT
 
 def smooth_step(x):
     if x <= 0.0:
@@ -52,6 +53,10 @@ def smooth_step(x):
         return(1.0)
     else:
         return(-2.0*x*x*x+3.0*x*x)
+        
+# Conversion of erf function into CDF format
+def cdf(x):
+    return((1.0+math.erf(x/math.sqrt(2)))/2)
 
 # -------- new step function 2018-09-16 ------
 def stepsWithPeriods(N, period, width, iCurr):
@@ -231,6 +236,13 @@ class Expression:
                 self.fargs = []
                 for arg in ast_exp.fargs:
                     self.fargs.append(Expression(arg, circuit, phase_dependence))
+            elif self.fname == 'rsjph':
+                if len(ast_exp.fargs) != 4:
+                    self.etype = ExpressionAST.EXP_TYPE_INVALID
+                    raise Exception("Invalid number of parameters for " + self.fname + " in " + str(circuit))
+                self.fargs = []
+                for arg in ast_exp.fargs:
+                    self.fargs.append(Expression(arg, circuit, phase_dependence))
             elif self.fname == 'tjm':
                 if len(ast_exp.fargs) != 6:
                     self.etype = ExpressionAST.EXP_TYPE_INVALID
@@ -270,12 +282,14 @@ class Expression:
                 self.fargs = [self.test_exp, self.true_exp, self.false_exp]
             elif self.fname == 'avert':
                 if len(ast_exp.fargs) > 1 and len(ast_exp.fargs) < 5:
-                    self.fargs = [Expression(ast_exp.fargs[0], circuit, phase_dependence)]
-                    arg_values = [0.0]
-                    for arg in ast_exp.fargs[1:]:
+                    self.fargs = [Expression(ast_exp.fargs[0], circuit, phase_dependence),
+                                  Expression(ast_exp.fargs[1], circuit, phase_dependence)]
+                    arg_values = [SP.INITIAL_RAMP, self.fargs[1].Value()]
+                    for arg in ast_exp.fargs[2:]:
                         exp = Expression(arg, circuit, phase_dependence)
                         arg_values.append(exp.Value())
-                    self.aver = AverT(*arg_values)
+                    self.avert = AverageT(*arg_values)
+                    self.avert.TaverVar=self.fargs[1]
                 else: 
                     self.etype = ExpressionAST.EXP_TYPE_INVALID
                     raise Exception("Invalid number of arguments for function avert() in " + str(circuit))
@@ -284,6 +298,40 @@ class Expression:
                     self.etype = ExpressionAST.EXP_TYPE_INVALID
                     raise Exception("Invalid parameters for " + self.fname + " in " + str(circuit))
                 self.fargs = [Expression(ast_exp.fargs[0], circuit, phase_dependence)]
+            elif self.fname == 'random':
+                self.fargs = []
+            elif self.fname == 'cdf':
+                if len(ast_exp.fargs) == 1:
+                    self.fargs = [Expression(ast_exp.fargs[0], circuit, phase_dependence)]
+                else:
+                    self.etype = ExpressionAST.EXP_TYPE_INVALID
+                    raise Exception("Invalid number of parameters for function cdf() in" + str(circuit))
+                if self.fargs[0].etype == ExpressionAST.EXP_TYPE_INVALID:
+                    self.etype = ExpressionAST.EXP_TYPE_INVALID
+                    raise Exception("Invalid parameter for function cdf() in " + str(circuit))
+            elif self.fname == 'setsmooth':
+                if len(ast_exp.fargs) == 3 and ast_exp.fargs[0].etype == ExpressionAST.EXP_TYPE_IDENT:
+                    ident = ast_exp.fargs[0].ident
+                    if ident in local_externals:
+                        self.set_param = local_externals[ident]
+                    elif ident in local_params:
+                        self.set_param = local_params[ident]
+                    elif ident in psglobals.Parameters:
+                        self.set_param = psglobals.Parameters[ident]
+                    else:
+                        self.etype = ExpressionAST.EXP_TYPE_INVALID
+                        raise Exception("Invalid first parameter for function setsmooth() in" + str(circuit))
+                       
+                    self.fargs = [Expression(ast_exp.fargs[1], circuit, phase_dependence),
+                                  Expression(ast_exp.fargs[2], circuit, phase_dependence)]
+                else:
+                    self.etype = ExpressionAST.EXP_TYPE_INVALID
+                    raise Exception("Invalid number of parameters for function setsmooth() in" + str(circuit))
+                    
+                if self.fargs[0].etype == ExpressionAST.EXP_TYPE_INVALID or \
+                    self.fargs[1].etype == ExpressionAST.EXP_TYPE_INVALID:
+                    self.etype = ExpressionAST.EXP_TYPE_INVALID
+                    raise Exception("Invalid parameters for function setsmooth() in " + str(circuit))
             elif self.fname in dir(math):
                 # One of the math module functions
                 self.func = getattr(math, self.fname)
@@ -601,9 +649,12 @@ class Expression:
             elif self.fname == 'round':
                 return(round(self.fargs[0].Value(),int(self.fargs[1].Value())))
             elif self.fname == 'integrate':
-                val = self.fargs[0].Value()
-                self.last_value += SP.TimeSteps[0] * (self.last_fvalue + val) / 2.0
-                self.last_fvalue = val
+                if psglobals.CurrentTime < SP.INITIAL_RAMP:
+                    self.last_value=0
+                else:
+                    val = self.fargs[0].Value()
+                    self.last_value += SP.TimeSteps[0] * (self.last_fvalue + val) / 2.0
+                    self.last_fvalue = val
                 return(self.last_value) 
             elif self.fname == 'if':
                 if self.test_exp.Value() == 1:
@@ -612,6 +663,13 @@ class Expression:
                     return self.false_exp.Value()
             elif self.fname == 'avert':
                 return self.avert.step(psglobals.CurrentTime, self.fargs[0].Value())
+            elif self.fname == 'random':
+                return random.random()
+            elif self.fname == 'cfd':
+                return cdf(self.fargs[0].Value())
+            elif self.fname == 'setsmooth':
+                self.set_param.SetValueSmooth(self.fargs[0].Value(), self.fargs[1].Value())
+                return(1.0)
             elif self.fname == 'pyeval':
                 return eval(self.fargs[0].sconst_val)
             elif self.func is not None:
